using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using NwndAdHcWaAPI.Domain;
using NwndAdHcWaAPI.BusinessObject;
using System.Net.Http;
using System.Text.Json;
using System.Threading.Tasks;

namespace NwndAdHcWa.Pages
{
     public class Categories_ListForeachModel : PageModel
     {
         public List<Categories> CategoriesData { get; set; }
         public string[,] CategoriesFieldNames { get; set; }
         public string FieldToSort { get; set; }
         public string FieldToSortWithOrder { get; set; }
         public string FieldSortOrder { get; set; }
         public int StartPage { get; set; }
         public int EndPage { get; set; }
         public int CurrentPage { get; set; }
         public int NumberOfPagesToShow { get; set; }
         public int TotalPages { get; set; }
         public List<string> UnsortableFields { get; set; }

         /// <summary>
         /// Initial handler the razor page encounters.
         /// </summary>
         public async Task OnGet()
         {
             string sidx = String.Empty;
             string sord = String.Empty;
             int currentPage = 1;

             await GetDataAsync(sidx, sord, currentPage);
         }

         /// <summary>
         /// Handler, deletes a record.
         /// </summary>
         public async Task<IActionResult> OnGetRemoveAsync(int id)
         {
             // delete data via web api
             HttpResponseMessage response = await Functions.HttpClientDeleteAsync("CategoriesApi/Delete/" + id);

             // check the status of the httpclient delete operation
             if (response != null)
             {
                 if (response.IsSuccessStatusCode)
                     return new JsonResult(true);
                 else
                     return BadRequest(response.ReasonPhrase);
             }

             return BadRequest();
         }

         public async Task OnGetGridData(string sidx, string sord, int? _page)
         {
             await GetDataAsync(sidx, sord, _page);
         }

         public async Task GetDataAsync(string sidx, string sord, int? _page)
         {
             // get the default number of rows to show
             int rows = Functions.GetGridNumberOfRows();

             // the numberOfPagesToShow is used to show the "< First..." link in the razor view
             int numberOfPagesToShow = Functions.GetGridNumberOfPagesToShow();

             // when _page is null, set it to 1
             int currentPage = _page is null ? 1 : Convert.ToInt32(_page);

             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((currentPage * rows) - rows);

             // get the total number of records
             string responseBody1 = await Functions.HttpClientGetAsync("CategoriesApi/GetRecordCount/");
             int totalRecords = JsonSerializer.Deserialize<int>(responseBody1);

             // calculate the total number of pages
             int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);

             // get records based on filters
             List<Categories> objCategoriesCol = null;
             string responseBody2 = await Functions.HttpClientGetAsync("CategoriesApi/SelectSkipAndTake/?rows=" + rows + "&page=" + currentPage + "&sidx=" + sidx + "&sord=" + sord);

             // make sure responseBody2 is not empty before deserialization
             if(!String.IsNullOrEmpty(responseBody2))
                 objCategoriesCol = JsonSerializer.Deserialize<List<Categories>>(responseBody2);

             // fields and titles
             string[,] fieldNames = new string[,] {
                 {"CategoryID", "Category ID"},
                 {"CategoryName", "Category Name"},
                 {"Description", "Description"}
             };

             List<string> unsortableFields = new List<string>();
             unsortableFields.Add("Description");

             // assign properties
             CategoriesData = objCategoriesCol;
             CategoriesFieldNames = fieldNames;
             TotalPages = totalPages;
             CurrentPage = currentPage;
             FieldToSort = String.IsNullOrEmpty(sidx) ? "CategoryID" : sidx;
             FieldSortOrder = String.IsNullOrEmpty(sord) ? "asc" : sord;
             FieldToSortWithOrder = String.IsNullOrEmpty(sidx) ? "CategoryID" : (sidx + " " + sord).Trim();
             NumberOfPagesToShow = numberOfPagesToShow;
             StartPage = Functions.GetPagerStartPage(currentPage, numberOfPagesToShow);
             EndPage = Functions.GetPagerEndPage(StartPage, numberOfPagesToShow, totalPages);
             UnsortableFields = unsortableFields;
         }
     }
}
