using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using NwndAdHcWaAPI.Domain;
using NwndAdHcWaAPI.BusinessObject;
using System.Net.Http;
using System.Text.Json;
using System.Threading.Tasks;

namespace NwndAdHcWa.Pages
{
     public class Categories_ListMultipleDeleteModel : PageModel
     {

         /// <summary>
         /// Handler, deletes multiple records based on the comma-delimited ids
         /// </summary>
         public async Task<IActionResult> OnGetRemoveMultipleAsync(string ids)
         {
             // delete multiple items via web api
             HttpResponseMessage response = await Functions.HttpClientDeleteAsync("CategoriesApi/DeleteMultiple/?ids=" + ids);

             // check the status of the httpclient delete multiple operation
             if (response != null)
             {
                 if (response.IsSuccessStatusCode)
                     return new JsonResult(true);
                 else
                     return BadRequest(response.ReasonPhrase);
             }

             return BadRequest();
         }

         /// <summary>
         /// GET: /Categories/GridData
         /// Gets the json needed by the jqgrid for use by the GridData
         /// </summary>
         public async Task<IActionResult> OnGetGridDataAsync(string sidx, string sord, int _page, int rows)
         {
             // get the total number of records
             string responseBody1 = await Functions.HttpClientGetAsync("CategoriesApi/GetRecordCount/");
             int totalRecords = JsonSerializer.Deserialize<int>(responseBody1);

             // get records
             List<Categories> objCategoriesCol = null;
             string responseBody2 = await Functions.HttpClientGetAsync("CategoriesApi/SelectSkipAndTake/?rows=" + rows + "&page=" + _page + "&sidx=" + sidx + "&sord=" + sord);

             // make sure responseBody2 is not empty before deserialization
             if(!String.IsNullOrEmpty(responseBody2))
                 objCategoriesCol = JsonSerializer.Deserialize<List<Categories>>(responseBody2);

             // calculate the total number of pages
             int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);

             // return a null in json for use by the jqgrid
             if (objCategoriesCol is null)
                 return new JsonResult("{ total = 0, page = 0, records = 0, rows = null }");

             // create a serialized json object for use by the jqgrid
             var jsonData = new
             {
                 total = totalPages,
                 _page,
                 records = totalRecords,
                 rows = (
                     from objCategories in objCategoriesCol
                     select new
                     {
                         id = objCategories.CategoryID,
                         cell = new string[] { 
                             objCategories.CategoryID.ToString(),
                             objCategories.CategoryName,
                             objCategories.Description
                         }
                     }).ToArray()
             };

             return new JsonResult(jsonData);
         }
     }
}
