using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using NwndAdHcWaAPI.Domain;
using NwndAdHcWaAPI.BusinessObject;
using NwndAdHcWa.PartialModels;
using System.Net.Http;
using System.Text.Json;
using System.Threading.Tasks;

namespace NwndAdHcWa.Pages
{
     public class Products_AddModel : PageModel
     {
         [BindProperty]
         public NwndAdHcWaAPI.BusinessObject.Products Products { get; set; }

         [BindProperty]
         public string ReturnUrl { get; set; }

         public AddEditProductsPartialModel PartialModel { get; set; }

         /// <summary>
         /// Initial handler the razor page encounters.
         /// </summary>
         public async Task OnGetAsync(string returnUrl)
         {
             await LoadPageAsync(returnUrl);
         }

         private async Task<PageResult> LoadPageAsync(string returnUrl)
         {
             // create the model used by the partial page
             AddEditProductsPartialModel model = new AddEditProductsPartialModel();
             model.SuppliersDropDownListData = await GetSuppliersDropDownListDataAsync();
             model.CategoriesDropDownListData = await GetCategoriesDropDownListDataAsync();
             model.Operation = CrudOperation.Add;
             model.ReturnUrl = returnUrl;

             // assign values to the model used by this page
             PartialModel = model;

             // assign the return url
             ReturnUrl = returnUrl;

             return Page();
         }

         public async Task<IActionResult> OnPostAddAsync()
         {
             if (ModelState.IsValid)
             {
                 try
                 {
                     // add new record
                     await ProductsFunctions.AddOrEditAsync(Products, CrudOperation.Add);
                     return RedirectToPage(ReturnUrl);
                 }
                 catch(Exception ex)
                 {
                     if (ex.InnerException != null)
                         ModelState.AddModelError("", ex.InnerException.Message);
                     else
                         ModelState.AddModelError("", ex.Message);
                 }
             }

             // if we got this far, something failed, redisplay form
             return await LoadPageAsync(ReturnUrl);
         }

         /// <summary>
         /// Selects records as a collection (List) of Suppliers
         /// Retrieves a collection (List) of two colums per row, normally the Primary Key column and another column for use in a drop down list
         /// </summary>
         private async Task<List<Suppliers>> GetSuppliersDropDownListDataAsync()
         {
             // get records
             string responseBody = await Functions.HttpClientGetAsync("SuppliersApi/SelectSuppliersDropDownListData/");
             List<Suppliers> objSuppliersCol = JsonSerializer.Deserialize<List<Suppliers>>(responseBody);

             return objSuppliersCol;
         }

         /// <summary>
         /// Selects records as a collection (List) of Categories
         /// Retrieves a collection (List) of two colums per row, normally the Primary Key column and another column for use in a drop down list
         /// </summary>
         private async Task<List<Categories>> GetCategoriesDropDownListDataAsync()
         {
             // get records
             string responseBody = await Functions.HttpClientGetAsync("CategoriesApi/SelectCategoriesDropDownListData/");
             List<Categories> objCategoriesCol = JsonSerializer.Deserialize<List<Categories>>(responseBody);

             return objCategoriesCol;
         }
     }
}
