using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using NwndAdHcWaAPI.Domain;
using NwndAdHcWaAPI.BusinessObject;
using System.Net.Http;
using System.Text.Json;
using System.Threading.Tasks;

namespace NwndAdHcWa.Pages
{
     public class Products_ListByCategoryIDModel : PageModel
     {
         [BindProperty]
         public List<Categories> CategoriesDropDownListData { get; set; }

         [BindProperty]
         public NwndAdHcWaAPI.Models.ProductsModel ProductsModel { get; set; }

         /// <summary>
         /// Initial handler the razor page encounters.
         /// </summary>
         public async Task OnGetAsync()
         {
             CategoriesDropDownListData = await GetCategoriesDropDownListDataAsync();
         }

         /// <summary>
         /// Selects records as a collection (List) of Suppliers
         /// Retrieves a collection (List) of two colums per row, normally the Primary Key column and another column for use in a drop down list
         /// </summary>
         private async Task<List<Suppliers>> GetSuppliersDropDownListDataAsync()
         {
             // get records
             string responseBody = await Functions.HttpClientGetAsync("SuppliersApi/SelectSuppliersDropDownListData/");
             List<Suppliers> objSuppliersCol = JsonSerializer.Deserialize<List<Suppliers>>(responseBody);

             return objSuppliersCol;
         }

         /// <summary>
         /// Selects records as a collection (List) of Categories
         /// Retrieves a collection (List) of two colums per row, normally the Primary Key column and another column for use in a drop down list
         /// </summary>
         private async Task<List<Categories>> GetCategoriesDropDownListDataAsync()
         {
             // get records
             string responseBody = await Functions.HttpClientGetAsync("CategoriesApi/SelectCategoriesDropDownListData/");
             List<Categories> objCategoriesCol = JsonSerializer.Deserialize<List<Categories>>(responseBody);

             return objCategoriesCol;
         }

         /// <summary>
         /// GET: /Products/GridDataByCategoryID
         /// Gets the json needed by the jqgrid for use by the GridDataByCategoryID
         /// </summary>
         public async Task<IActionResult> OnGetGridDataByCategoryIDAsync(string sidx, string sord, int _page, int rows, int? categoryID)
         {
             // get the total number of records
             string responseBody1 = await Functions.HttpClientGetAsync("ProductsApi/GetRecordCountByCategoryID/" + categoryID);
             int totalRecords = JsonSerializer.Deserialize<int>(responseBody1);

             // get records
             List<Products> objProductsCol = null;
             string responseBody2 = await Functions.HttpClientGetAsync("ProductsApi/SelectSkipAndTakeByCategoryID/?rows=" + rows + "&page=" + _page + "&sidx=" + sidx + "&sord=" + sord + "&categoryID=" + categoryID);

             // make sure responseBody2 is not empty before deserialization
             if(!String.IsNullOrEmpty(responseBody2))
                 objProductsCol = JsonSerializer.Deserialize<List<Products>>(responseBody2);

             // calculate the total number of pages
             int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);

             // return a null in json for use by the jqgrid
             if (objProductsCol is null)
                 return new JsonResult("{ total = 0, page = 0, records = 0, rows = null }");

             // create a serialized json object for use by the jqgrid
             var jsonData = new
             {
                 total = totalPages,
                 _page,
                 records = totalRecords,
                 rows = (
                     from objProducts in objProductsCol
                     select new
                     {
                         id = objProducts.ProductID,
                         cell = new string[] { 
                             objProducts.ProductID.ToString(),
                             objProducts.ProductName,
                             objProducts.SupplierID.HasValue ? objProducts.SupplierID.Value.ToString() : "",
                             objProducts.CategoryID.HasValue ? objProducts.CategoryID.Value.ToString() : "",
                             objProducts.QuantityPerUnit,
                             objProducts.UnitPrice.HasValue ? objProducts.UnitPrice.Value.ToString() : "",
                             objProducts.UnitsInStock.HasValue ? objProducts.UnitsInStock.Value.ToString() : "",
                             objProducts.UnitsOnOrder.HasValue ? objProducts.UnitsOnOrder.Value.ToString() : "",
                             objProducts.ReorderLevel.HasValue ? objProducts.ReorderLevel.Value.ToString() : "",
                             objProducts.Discontinued.ToString()
                         }
                     }).ToArray()
             };

             return new JsonResult(jsonData);
         }
     }
}
