using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using NwndAdHcWaAPI.Domain;
using NwndAdHcWaAPI.BusinessObject;
using System.Net.Http;
using System.Text.Json;
using System.Threading.Tasks;

namespace NwndAdHcWa.Pages
{
     public class Products_ListMasterDetailSubGridByCategoryIDModel : PageModel
     {

         /// <summary>
         /// GET: /Categories/GridData
         /// Gets the json needed by the jqgrid for use by the GridData
         /// </summary>
         public async Task<IActionResult> OnGetGridDataAsync(string sidx, string sord, int _page, int rows)
         {
             // get the total number of records
             string responseBody1 = await Functions.HttpClientGetAsync("CategoriesApi/GetRecordCount/");
             int totalRecords = JsonSerializer.Deserialize<int>(responseBody1);

             // get records
             List<Categories> objCategoriesCol = null;
             string responseBody2 = await Functions.HttpClientGetAsync("CategoriesApi/SelectSkipAndTake/?rows=" + rows + "&page=" + _page + "&sidx=" + sidx + "&sord=" + sord);

             // make sure responseBody2 is not empty before deserialization
             if(!String.IsNullOrEmpty(responseBody2))
                 objCategoriesCol = JsonSerializer.Deserialize<List<Categories>>(responseBody2);

             // calculate the total number of pages
             int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);

             // return a null in json for use by the jqgrid
             if (objCategoriesCol is null)
                 return new JsonResult("{ total = 0, page = 0, records = 0, rows = null }");

             // create a serialized json object for use by the jqgrid
             var jsonData = new
             {
                 total = totalPages,
                 _page,
                 records = totalRecords,
                 rows = (
                     from objCategories in objCategoriesCol
                     select new
                     {
                         id = objCategories.CategoryID,
                         cell = new string[] { 
                             objCategories.CategoryID.ToString(),
                             objCategories.CategoryName,
                             objCategories.Description
                         }
                     }).ToArray()
             };

             return new JsonResult(jsonData);
         }

         /// <summary>
         /// GET: /Products/GridDataWithFilters
         /// Gets the json needed by the jqgrid for use by the GridDataWithFilters
         /// </summary>
         public async Task<IActionResult> OnGetGridDataWithFiltersAsync(string _search, string nd, int rows, int _page, string sidx, string sord, string filters = "")
         {
             int? productID = null;
             string productName = String.Empty;
             int? supplierID = null;
             int? categoryID = null;
             string quantityPerUnit = String.Empty;
             decimal? unitPrice = null;
             Int16? unitsInStock = null;
             Int16? unitsOnOrder = null;
             Int16? reorderLevel = null;
             bool? discontinued = null;

             if (!String.IsNullOrEmpty(filters))
             {
                 // deserialize filters and get values being searched
                 var jsonResult = Newtonsoft.Json.JsonConvert.DeserializeObject<Dictionary<string, dynamic>>(filters);

                 foreach (var rule in jsonResult["rules"])
                 {
                     if (rule["field"].Value.ToLower() == "productid")
                         productID = Convert.ToInt32(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "productname")
                         productName = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "supplierid")
                         supplierID = Convert.ToInt32(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "categoryid")
                         categoryID = Convert.ToInt32(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "quantityperunit")
                         quantityPerUnit = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "unitprice")
                         unitPrice = Convert.ToDecimal(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "unitsinstock")
                         unitsInStock = Convert.ToInt16(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "unitsonorder")
                         unitsOnOrder = Convert.ToInt16(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "reorderlevel")
                         reorderLevel = Convert.ToInt16(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "discontinued")
                         discontinued = Convert.ToBoolean(rule["data"].Value);

                 }

                 // sometimes jqgrid assigns a -1 to numeric fields when no value is assigned
                 // instead of assigning a null, we'll correct this here
                 if (productID == -1)
                     productID = null;

                 if (supplierID == -1)
                     supplierID = null;

                 if (categoryID == -1)
                     categoryID = null;

                 if (unitPrice == -1)
                     unitPrice = null;

                 if (unitsInStock == -1)
                     unitsInStock = null;

                 if (unitsOnOrder == -1)
                     unitsOnOrder = null;

                 if (reorderLevel == -1)
                     reorderLevel = null;

             }

             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((_page * rows) - rows);

             // get the total number of records based on filters
             int totalRecords = 0;
             string responseBody1 = await Functions.HttpClientGetAsync("ProductsApi/GetRecordCountDynamicWhere/?productID=" + productID + "&productName=" + productName + "&supplierID=" + supplierID + "&categoryID=" + categoryID + "&quantityPerUnit=" + quantityPerUnit + "&unitPrice=" + unitPrice + "&unitsInStock=" + unitsInStock + "&unitsOnOrder=" + unitsOnOrder + "&reorderLevel=" + reorderLevel + "&discontinued=" + discontinued);
             totalRecords = JsonSerializer.Deserialize<int>(responseBody1);

             // get records based on filters
             List<Products> objProductsCol = null;
             string responseBody2 = await Functions.HttpClientGetAsync("ProductsApi/SelectSkipAndTakeDynamicWhere/?productID=" + productID + "&productName=" + productName + "&supplierID=" + supplierID + "&categoryID=" + categoryID + "&quantityPerUnit=" + quantityPerUnit + "&unitPrice=" + unitPrice + "&unitsInStock=" + unitsInStock + "&unitsOnOrder=" + unitsOnOrder + "&reorderLevel=" + reorderLevel + "&discontinued=" + discontinued + "&rows=" + rows + "&startRowIndex=" + startRowIndex + "&sortByExpression=" + sidx + " " + sord);

             // make sure responseBody2 is not empty before deserialization
             if(!String.IsNullOrEmpty(responseBody2))
                 objProductsCol = JsonSerializer.Deserialize<List<Products>>(responseBody2);

             // calculate the total number of pages
             int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);

             // return a null in json for use by the jqgrid
             if (objProductsCol is null)
                 return new JsonResult("{ total = 0, page = 0, records = 0, rows = null }");

             // create a serialized json object for use by the jqgrid
             var jsonData = new
             {
                 total = totalPages,
                 _page,
                 records = totalRecords,
                 rows = (
                     from objProducts in objProductsCol
                     select new
                     {
                         id = objProducts.ProductID,
                         cell = new string[] { 
                             objProducts.ProductID.ToString(),
                             objProducts.ProductName,
                             objProducts.SupplierID.HasValue ? objProducts.SupplierID.Value.ToString() : "",
                             objProducts.CategoryID.HasValue ? objProducts.CategoryID.Value.ToString() : "",
                             objProducts.QuantityPerUnit,
                             objProducts.UnitPrice.HasValue ? objProducts.UnitPrice.Value.ToString() : "",
                             objProducts.UnitsInStock.HasValue ? objProducts.UnitsInStock.Value.ToString() : "",
                             objProducts.UnitsOnOrder.HasValue ? objProducts.UnitsOnOrder.Value.ToString() : "",
                             objProducts.ReorderLevel.HasValue ? objProducts.ReorderLevel.Value.ToString() : "",
                             objProducts.Discontinued.ToString()
                         }
                     }).ToArray()
             };

             return new JsonResult(jsonData);
         }
     }
}
