using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using NwndAdHcWaAPI.Domain;
using NwndAdHcWaAPI.BusinessObject;
using System.Net.Http;
using System.Text.Json;
using System.Threading.Tasks;

namespace NwndAdHcWa.Pages
{
     public class Suppliers_ListCrudRedirectModel : PageModel
     {

         /// <summary>
         /// Handler, deletes a record.
         /// </summary>
         public async Task<IActionResult> OnGetRemoveAsync(int id)
         {
             // delete data via web api
             HttpResponseMessage response = await Functions.HttpClientDeleteAsync("SuppliersApi/Delete/" + id);

             // check the status of the httpclient delete operation
             if (response != null)
             {
                 if (response.IsSuccessStatusCode)
                     return new JsonResult(true);
                 else
                     return BadRequest(response.ReasonPhrase);
             }

             return BadRequest();
         }

         /// <summary>
         /// GET: /Suppliers/GridData
         /// Gets the json needed by the jqgrid for use by the GridData
         /// </summary>
         public async Task<IActionResult> OnGetGridDataAsync(string sidx, string sord, int _page, int rows)
         {
             // get the total number of records
             string responseBody1 = await Functions.HttpClientGetAsync("SuppliersApi/GetRecordCount/");
             int totalRecords = JsonSerializer.Deserialize<int>(responseBody1);

             // get records
             List<Suppliers> objSuppliersCol = null;
             string responseBody2 = await Functions.HttpClientGetAsync("SuppliersApi/SelectSkipAndTake/?rows=" + rows + "&page=" + _page + "&sidx=" + sidx + "&sord=" + sord);

             // make sure responseBody2 is not empty before deserialization
             if(!String.IsNullOrEmpty(responseBody2))
                 objSuppliersCol = JsonSerializer.Deserialize<List<Suppliers>>(responseBody2);

             // calculate the total number of pages
             int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);

             // return a null in json for use by the jqgrid
             if (objSuppliersCol is null)
                 return new JsonResult("{ total = 0, page = 0, records = 0, rows = null }");

             // create a serialized json object for use by the jqgrid
             var jsonData = new
             {
                 total = totalPages,
                 _page,
                 records = totalRecords,
                 rows = (
                     from objSuppliers in objSuppliersCol
                     select new
                     {
                         id = objSuppliers.SupplierID,
                         cell = new string[] { 
                             objSuppliers.SupplierID.ToString(),
                             objSuppliers.CompanyName,
                             objSuppliers.ContactName,
                             objSuppliers.ContactTitle,
                             objSuppliers.Address,
                             objSuppliers.City,
                             objSuppliers.Region1,
                             objSuppliers.PostalCode,
                             objSuppliers.Country,
                             objSuppliers.Phone,
                             objSuppliers.Fax,
                             objSuppliers.HomePage
                         }
                     }).ToArray()
             };

             return new JsonResult(jsonData);
         }
     }
}
