using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using NwndAdHcWaAPI.Domain;
using NwndAdHcWaAPI.BusinessObject;
using System.Net.Http;
using System.Text.Json;
using System.Threading.Tasks;

namespace NwndAdHcWa.Pages
{
     public class Suppliers_ListSearchModel : PageModel
     {
         /// <summary>
         /// Initial handler the razor page encounters.
         /// </summary>
         public void OnGet()
         {
         }

         /// <summary>
         /// GET: /Suppliers/GridDataWithFilters
         /// Gets the json needed by the jqgrid for use by the GridDataWithFilters
         /// </summary>
         public async Task<IActionResult> OnGetGridDataWithFiltersAsync(string _search, string nd, int rows, int _page, string sidx, string sord, string filters = "")
         {
             int? supplierID = null;
             string companyName = String.Empty;
             string contactName = String.Empty;
             string contactTitle = String.Empty;
             string address = String.Empty;
             string city = String.Empty;
             string region1 = String.Empty;
             string postalCode = String.Empty;
             string country = String.Empty;
             string phone = String.Empty;
             string fax = String.Empty;

             if (!String.IsNullOrEmpty(filters))
             {
                 // deserialize filters and get values being searched
                 var jsonResult = Newtonsoft.Json.JsonConvert.DeserializeObject<Dictionary<string, dynamic>>(filters);

                 foreach (var rule in jsonResult["rules"])
                 {
                     if (rule["field"].Value.ToLower() == "supplierid")
                         supplierID = Convert.ToInt32(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "companyname")
                         companyName = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "contactname")
                         contactName = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "contacttitle")
                         contactTitle = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "address")
                         address = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "city")
                         city = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "region1")
                         region1 = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "postalcode")
                         postalCode = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "country")
                         country = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "phone")
                         phone = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "fax")
                         fax = rule["data"].Value;

                 }

                 // sometimes jqgrid assigns a -1 to numeric fields when no value is assigned
                 // instead of assigning a null, we'll correct this here
                 if (supplierID == -1)
                     supplierID = null;

             }

             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((_page * rows) - rows);

             // get the total number of records based on filters
             int totalRecords = 0;
             string responseBody1 = await Functions.HttpClientGetAsync("SuppliersApi/GetRecordCountDynamicWhere/?supplierID=" + supplierID + "&companyName=" + companyName + "&contactName=" + contactName + "&contactTitle=" + contactTitle + "&address=" + address + "&city=" + city + "&region1=" + region1 + "&postalCode=" + postalCode + "&country=" + country + "&phone=" + phone + "&fax=" + fax);
             totalRecords = JsonSerializer.Deserialize<int>(responseBody1);

             // get records based on filters
             List<Suppliers> objSuppliersCol = null;
             string responseBody2 = await Functions.HttpClientGetAsync("SuppliersApi/SelectSkipAndTakeDynamicWhere/?supplierID=" + supplierID + "&companyName=" + companyName + "&contactName=" + contactName + "&contactTitle=" + contactTitle + "&address=" + address + "&city=" + city + "&region1=" + region1 + "&postalCode=" + postalCode + "&country=" + country + "&phone=" + phone + "&fax=" + fax + "&rows=" + rows + "&startRowIndex=" + startRowIndex + "&sortByExpression=" + sidx + " " + sord);

             // make sure responseBody2 is not empty before deserialization
             if(!String.IsNullOrEmpty(responseBody2))
                 objSuppliersCol = JsonSerializer.Deserialize<List<Suppliers>>(responseBody2);

             // calculate the total number of pages
             int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);

             // return a null in json for use by the jqgrid
             if (objSuppliersCol is null)
                 return new JsonResult("{ total = 0, page = 0, records = 0, rows = null }");

             // create a serialized json object for use by the jqgrid
             var jsonData = new
             {
                 total = totalPages,
                 _page,
                 records = totalRecords,
                 rows = (
                     from objSuppliers in objSuppliersCol
                     select new
                     {
                         id = objSuppliers.SupplierID,
                         cell = new string[] { 
                             objSuppliers.SupplierID.ToString(),
                             objSuppliers.CompanyName,
                             objSuppliers.ContactName,
                             objSuppliers.ContactTitle,
                             objSuppliers.Address,
                             objSuppliers.City,
                             objSuppliers.Region1,
                             objSuppliers.PostalCode,
                             objSuppliers.Country,
                             objSuppliers.Phone,
                             objSuppliers.Fax,
                             objSuppliers.HomePage
                         }
                     }).ToArray()
             };

             return new JsonResult(jsonData);
         }
     }
}
