using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using NwndAdHcWaAPI.Domain;
using NwndAdHcWaAPI.BusinessObject;
using NwndAdHcWa.PartialModels;
using System.Net.Http;
using System.Text.Json;
using System.Threading.Tasks;

namespace NwndAdHcWa.Pages
{
     public class Suppliers_UpdateModel : PageModel
     {
         [BindProperty]
         public NwndAdHcWaAPI.BusinessObject.Suppliers Suppliers { get; set; }

         [BindProperty]
         public string ReturnUrl { get; set; }

         public AddEditSuppliersPartialModel PartialModel { get; set; }

         /// <summary>
         /// Initial handler the razor page encounters.
         /// </summary>
         public async Task OnGetAsync(int id, string returnUrl)
         {
             await LoadPageAsync(id, returnUrl);
         }

         public async Task<PageResult> LoadPageAsync(int id, string returnUrl)
         {
             // select a record by primary key(s)
             NwndAdHcWaAPI.BusinessObject.Suppliers objSuppliers = await GetSuppliersByPrimaryKeyAsync(id);

             // create the model used by the partial page
             AddEditSuppliersPartialModel model = new AddEditSuppliersPartialModel();
             model.Operation = CrudOperation.Update;
             model.ReturnUrl = returnUrl;
             model.Suppliers = objSuppliers;

             // assign values to the model used by this page
             PartialModel = model;

             // assign the return url
             ReturnUrl = returnUrl;

             return Page();
         }

         public async Task<IActionResult> OnPostUpdateAsync()
         {
             if (ModelState.IsValid)
             {
                 try
                 {
                     // update record
                     await SuppliersFunctions.AddOrEditAsync(Suppliers, CrudOperation.Update);
                     return RedirectToPage(ReturnUrl);
                 }
                 catch(Exception ex)
                 {
                     if (ex.InnerException != null)
                         ModelState.AddModelError("", ex.InnerException.Message);
                     else
                         ModelState.AddModelError("", ex.Message);
                 }
             }

             // if we got this far, something failed, redisplay form
             return await LoadPageAsync(Suppliers.SupplierID, ReturnUrl);
         }

         private async Task<Suppliers> GetSuppliersByPrimaryKeyAsync(int supplierID)
         {
             string responseBody = await Functions.HttpClientGetAsync("SuppliersApi/SelectByPrimaryKey/" + supplierID);
             Suppliers objSuppliers = JsonSerializer.Deserialize<Suppliers>(responseBody);

             return objSuppliers;
         }
     }
}
