using System;
using NwndAdHcWaAPI.DataLayer;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace NwndAdHcWaAPI.BusinessObject.Base
{
     /// <summary>
     /// Base class for Categories.  Do not make changes to this class,
     /// instead, put additional code in the Categories (parent) class.
     /// </summary>
     public class CategoriesBase : NwndAdHcWaAPI.Models.CategoriesModel
     {

         /// <summary>
         /// Constructor
         /// </summary>
         public CategoriesBase()
         {
         }

         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         public static async Task<Categories> SelectByPrimaryKeyAsync(int categoryID)
         {
             return await CategoriesDataLayer.SelectByPrimaryKeyAsync(categoryID);
         }

         /// <summary>
         /// Gets the total number of records in the Categories table
         /// </summary>
         public static async Task<int> GetRecordCountAsync()
         {
             return await CategoriesDataLayer.GetRecordCountAsync();
         }

         /// <summary>
         /// Gets the total number of records in the Categories table based on search parameters
         /// </summary>
         public static async Task<int> GetRecordCountDynamicWhereAsync(int? categoryID, string categoryName)
         {
             return await CategoriesDataLayer.GetRecordCountDynamicWhereAsync(categoryID, categoryName);
         }

         /// <summary>
         /// Selects records as a collection (List) of Categories sorted by the sortByExpression.
         /// </summary>
         public static async Task<List<Categories>> SelectSkipAndTakeAsync(int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = GetSortExpression(sortByExpression);
             return await CategoriesDataLayer.SelectSkipAndTakeAsync(sortByExpression, startRowIndex, rows);
         }

         /// <summary>
         /// Selects records as a collection (List) of Categories sorted by the sortByExpression starting from the startRowIndex, based on the search parameters
         /// </summary>
         public static async Task<List<Categories>> SelectSkipAndTakeDynamicWhereAsync(int? categoryID, string categoryName, int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = GetSortExpression(sortByExpression);
             return await CategoriesDataLayer.SelectSkipAndTakeDynamicWhereAsync(categoryID, categoryName, sortByExpression, startRowIndex, rows);
         }

         /// <summary>
         /// Selects all records as a collection (List) of Categories
         /// </summary>
         public static async Task<List<Categories>> SelectAllAsync()
         {
             return await CategoriesDataLayer.SelectAllAsync();
         }

         /// <summary>
         /// Selects all records as a collection (List) of Categories sorted by the sort expression
         /// </summary>
         public static async Task<List<Categories>> SelectAllAsync(string sortByExpression)
         {
             List<Categories> objCategoriesCol = await CategoriesDataLayer.SelectAllAsync();
             return await SortByExpressionAsync(objCategoriesCol, sortByExpression);
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Categories.
         /// </summary>
         public static async Task<List<Categories>> SelectAllDynamicWhereAsync(int? categoryID, string categoryName)
         {
             return await CategoriesDataLayer.SelectAllDynamicWhereAsync(categoryID, categoryName);
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Categories sorted by the sort expression.
         /// </summary>
         public static async Task<List<Categories>> SelectAllDynamicWhereAsync(int? categoryID, string categoryName, string sortByExpression)
         {
             List<Categories> objCategoriesCol = await CategoriesDataLayer.SelectAllDynamicWhereAsync(categoryID, categoryName);
             return await SortByExpressionAsync(objCategoriesCol, sortByExpression);
         }

         /// <summary>
         /// Selects CategoryID and CategoryName columns for use with a DropDownList web control, ComboBox, CheckedBoxList, ListView, ListBox, etc
         /// </summary>
         public static async Task<List<Categories>> SelectCategoriesDropDownListDataAsync()
         {
             return await CategoriesDataLayer.SelectCategoriesDropDownListDataAsync();
         }

         /// <summary>
         /// Sorts the List<Categories >by sort expression
         /// </summary>
         public static async Task<List<Categories>> SortByExpressionAsync(List<Categories> objCategoriesCol, string sortExpression)
         {
             bool isSortDescending = sortExpression.ToLower().Contains(" desc");

             if (isSortDescending)
             {
                 sortExpression = sortExpression.Replace(" DESC", "");
                 sortExpression = sortExpression.Replace(" desc", "");
             }
             else
             {
                 sortExpression = sortExpression.Replace(" ASC", "");
                 sortExpression = sortExpression.Replace(" asc", "");
             }

             switch (sortExpression)
             {
                 case "CategoryID":
                     await Task.Run(() => objCategoriesCol.Sort(NwndAdHcWaAPI.BusinessObject.Categories.ByCategoryID));
                     break;
                 case "CategoryName":
                     await Task.Run(() => objCategoriesCol.Sort(NwndAdHcWaAPI.BusinessObject.Categories.ByCategoryName));
                     break;
                 case "Description":
                     await Task.Run(() => objCategoriesCol.Sort(NwndAdHcWaAPI.BusinessObject.Categories.ByDescription));
                     break;
                 default:
                     break;
             }

             if (isSortDescending)
                 objCategoriesCol.Reverse();

             return objCategoriesCol;
         }

         /// <summary>
         /// Inserts a record
         /// </summary>
         public async Task<int> InsertAsync()
         {
             Categories objCategories = (Categories)this;
             return await CategoriesDataLayer.InsertAsync(objCategories);
         }

         /// <summary>
         /// Updates a record
         /// </summary>
         public async Task UpdateAsync()
         {
             Categories objCategories = (Categories)this;
             await CategoriesDataLayer.UpdateAsync(objCategories);
         }

         /// <summary>
         /// Deletes a record based on primary key(s)
         /// </summary>
         public static async Task DeleteAsync(int categoryID)
         {
             await Task.Run(() => CategoriesDataLayer.DeleteAsync(categoryID));
         }

         /// <summary>
         /// Deletes multiple records based on primary keys
         /// </summary>
         public static async Task DeleteMultipleAsync(List<Int32> categoryIDList)
         {
             foreach (var id in categoryIDList)
             {
                 await CategoriesDataLayer.DeleteAsync(id);
             }
         }

         /// <summary>
         /// Gets the default sort expression when no sort expression is provided
         /// </summary>
         private static string GetSortExpression(string sortByExpression)
         {
             // when no sort expression is provided, CategoryID is set as the default in ascending order
             // for ascending order, "asc" is not needed, so it is removed
             if (String.IsNullOrEmpty(sortByExpression) || sortByExpression == " asc")
                 sortByExpression = "CategoryID";
             else if (sortByExpression.Contains(" asc"))
                 sortByExpression = sortByExpression.Replace(" asc", "");

             return sortByExpression;
         }

         /// <summary>
         /// Compares CategoryID used for sorting
         /// </summary>
         public static Comparison<Categories> ByCategoryID = delegate(Categories x, Categories y)
         {
             return x.CategoryID.CompareTo(y.CategoryID);
         };

         /// <summary>
         /// Compares CategoryName used for sorting
         /// </summary>
         public static Comparison<Categories> ByCategoryName = delegate(Categories x, Categories y)
         {
             string value1 = x.CategoryName ?? String.Empty;
             string value2 = y.CategoryName ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares Description used for sorting
         /// </summary>
         public static Comparison<Categories> ByDescription = delegate(Categories x, Categories y)
         {
             string value1 = x.Description ?? String.Empty;
             string value2 = y.Description ?? String.Empty;
             return value1.CompareTo(value2);
         };

     }
}
