using System;
using NwndAdHcWaAPI.DataLayer;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace NwndAdHcWaAPI.BusinessObject.Base
{
     /// <summary>
     /// Base class for Suppliers.  Do not make changes to this class,
     /// instead, put additional code in the Suppliers (parent) class.
     /// </summary>
     public class SuppliersBase : NwndAdHcWaAPI.Models.SuppliersModel
     {

         /// <summary>
         /// Constructor
         /// </summary>
         public SuppliersBase()
         {
         }

         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         public static async Task<Suppliers> SelectByPrimaryKeyAsync(int supplierID)
         {
             return await SuppliersDataLayer.SelectByPrimaryKeyAsync(supplierID);
         }

         /// <summary>
         /// Gets the total number of records in the Suppliers table
         /// </summary>
         public static async Task<int> GetRecordCountAsync()
         {
             return await SuppliersDataLayer.GetRecordCountAsync();
         }

         /// <summary>
         /// Gets the total number of records in the Suppliers table based on search parameters
         /// </summary>
         public static async Task<int> GetRecordCountDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax)
         {
             return await SuppliersDataLayer.GetRecordCountDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);
         }

         /// <summary>
         /// Selects records as a collection (List) of Suppliers sorted by the sortByExpression.
         /// </summary>
         public static async Task<List<Suppliers>> SelectSkipAndTakeAsync(int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = GetSortExpression(sortByExpression);
             return await SuppliersDataLayer.SelectSkipAndTakeAsync(sortByExpression, startRowIndex, rows);
         }

         /// <summary>
         /// Selects records as a collection (List) of Suppliers sorted by the sortByExpression starting from the startRowIndex, based on the search parameters
         /// </summary>
         public static async Task<List<Suppliers>> SelectSkipAndTakeDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax, int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = GetSortExpression(sortByExpression);
             return await SuppliersDataLayer.SelectSkipAndTakeDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax, sortByExpression, startRowIndex, rows);
         }

         /// <summary>
         /// Selects all records as a collection (List) of Suppliers
         /// </summary>
         public static async Task<List<Suppliers>> SelectAllAsync()
         {
             return await SuppliersDataLayer.SelectAllAsync();
         }

         /// <summary>
         /// Selects all records as a collection (List) of Suppliers sorted by the sort expression
         /// </summary>
         public static async Task<List<Suppliers>> SelectAllAsync(string sortByExpression)
         {
             List<Suppliers> objSuppliersCol = await SuppliersDataLayer.SelectAllAsync();
             return await SortByExpressionAsync(objSuppliersCol, sortByExpression);
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Suppliers.
         /// </summary>
         public static async Task<List<Suppliers>> SelectAllDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax)
         {
             return await SuppliersDataLayer.SelectAllDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Suppliers sorted by the sort expression.
         /// </summary>
         public static async Task<List<Suppliers>> SelectAllDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax, string sortByExpression)
         {
             List<Suppliers> objSuppliersCol = await SuppliersDataLayer.SelectAllDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);
             return await SortByExpressionAsync(objSuppliersCol, sortByExpression);
         }

         /// <summary>
         /// Selects SupplierID and CompanyName columns for use with a DropDownList web control, ComboBox, CheckedBoxList, ListView, ListBox, etc
         /// </summary>
         public static async Task<List<Suppliers>> SelectSuppliersDropDownListDataAsync()
         {
             return await SuppliersDataLayer.SelectSuppliersDropDownListDataAsync();
         }

         /// <summary>
         /// Sorts the List<Suppliers >by sort expression
         /// </summary>
         public static async Task<List<Suppliers>> SortByExpressionAsync(List<Suppliers> objSuppliersCol, string sortExpression)
         {
             bool isSortDescending = sortExpression.ToLower().Contains(" desc");

             if (isSortDescending)
             {
                 sortExpression = sortExpression.Replace(" DESC", "");
                 sortExpression = sortExpression.Replace(" desc", "");
             }
             else
             {
                 sortExpression = sortExpression.Replace(" ASC", "");
                 sortExpression = sortExpression.Replace(" asc", "");
             }

             switch (sortExpression)
             {
                 case "SupplierID":
                     await Task.Run(() => objSuppliersCol.Sort(NwndAdHcWaAPI.BusinessObject.Suppliers.BySupplierID));
                     break;
                 case "CompanyName":
                     await Task.Run(() => objSuppliersCol.Sort(NwndAdHcWaAPI.BusinessObject.Suppliers.ByCompanyName));
                     break;
                 case "ContactName":
                     await Task.Run(() => objSuppliersCol.Sort(NwndAdHcWaAPI.BusinessObject.Suppliers.ByContactName));
                     break;
                 case "ContactTitle":
                     await Task.Run(() => objSuppliersCol.Sort(NwndAdHcWaAPI.BusinessObject.Suppliers.ByContactTitle));
                     break;
                 case "Address":
                     await Task.Run(() => objSuppliersCol.Sort(NwndAdHcWaAPI.BusinessObject.Suppliers.ByAddress));
                     break;
                 case "City":
                     await Task.Run(() => objSuppliersCol.Sort(NwndAdHcWaAPI.BusinessObject.Suppliers.ByCity));
                     break;
                 case "Region1":
                     await Task.Run(() => objSuppliersCol.Sort(NwndAdHcWaAPI.BusinessObject.Suppliers.ByRegion1));
                     break;
                 case "PostalCode":
                     await Task.Run(() => objSuppliersCol.Sort(NwndAdHcWaAPI.BusinessObject.Suppliers.ByPostalCode));
                     break;
                 case "Country":
                     await Task.Run(() => objSuppliersCol.Sort(NwndAdHcWaAPI.BusinessObject.Suppliers.ByCountry));
                     break;
                 case "Phone":
                     await Task.Run(() => objSuppliersCol.Sort(NwndAdHcWaAPI.BusinessObject.Suppliers.ByPhone));
                     break;
                 case "Fax":
                     await Task.Run(() => objSuppliersCol.Sort(NwndAdHcWaAPI.BusinessObject.Suppliers.ByFax));
                     break;
                 case "HomePage":
                     await Task.Run(() => objSuppliersCol.Sort(NwndAdHcWaAPI.BusinessObject.Suppliers.ByHomePage));
                     break;
                 default:
                     break;
             }

             if (isSortDescending)
                 objSuppliersCol.Reverse();

             return objSuppliersCol;
         }

         /// <summary>
         /// Inserts a record
         /// </summary>
         public async Task<int> InsertAsync()
         {
             Suppliers objSuppliers = (Suppliers)this;
             return await SuppliersDataLayer.InsertAsync(objSuppliers);
         }

         /// <summary>
         /// Updates a record
         /// </summary>
         public async Task UpdateAsync()
         {
             Suppliers objSuppliers = (Suppliers)this;
             await SuppliersDataLayer.UpdateAsync(objSuppliers);
         }

         /// <summary>
         /// Deletes a record based on primary key(s)
         /// </summary>
         public static async Task DeleteAsync(int supplierID)
         {
             await Task.Run(() => SuppliersDataLayer.DeleteAsync(supplierID));
         }

         /// <summary>
         /// Deletes multiple records based on primary keys
         /// </summary>
         public static async Task DeleteMultipleAsync(List<Int32> supplierIDList)
         {
             foreach (var id in supplierIDList)
             {
                 await SuppliersDataLayer.DeleteAsync(id);
             }
         }

         /// <summary>
         /// Gets the default sort expression when no sort expression is provided
         /// </summary>
         private static string GetSortExpression(string sortByExpression)
         {
             // when no sort expression is provided, SupplierID is set as the default in ascending order
             // for ascending order, "asc" is not needed, so it is removed
             if (String.IsNullOrEmpty(sortByExpression) || sortByExpression == " asc")
                 sortByExpression = "SupplierID";
             else if (sortByExpression.Contains(" asc"))
                 sortByExpression = sortByExpression.Replace(" asc", "");

             return sortByExpression;
         }

         /// <summary>
         /// Compares SupplierID used for sorting
         /// </summary>
         public static Comparison<Suppliers> BySupplierID = delegate(Suppliers x, Suppliers y)
         {
             return x.SupplierID.CompareTo(y.SupplierID);
         };

         /// <summary>
         /// Compares CompanyName used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByCompanyName = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.CompanyName ?? String.Empty;
             string value2 = y.CompanyName ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares ContactName used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByContactName = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.ContactName ?? String.Empty;
             string value2 = y.ContactName ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares ContactTitle used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByContactTitle = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.ContactTitle ?? String.Empty;
             string value2 = y.ContactTitle ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares Address used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByAddress = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.Address ?? String.Empty;
             string value2 = y.Address ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares City used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByCity = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.City ?? String.Empty;
             string value2 = y.City ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares Region1 used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByRegion1 = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.Region1 ?? String.Empty;
             string value2 = y.Region1 ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares PostalCode used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByPostalCode = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.PostalCode ?? String.Empty;
             string value2 = y.PostalCode ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares Country used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByCountry = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.Country ?? String.Empty;
             string value2 = y.Country ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares Phone used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByPhone = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.Phone ?? String.Empty;
             string value2 = y.Phone ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares Fax used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByFax = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.Fax ?? String.Empty;
             string value2 = y.Fax ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares HomePage used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByHomePage = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.HomePage ?? String.Empty;
             string value2 = y.HomePage ?? String.Empty;
             return value1.CompareTo(value2);
         };

     }
}
