using System;
using System.Data;
using System.Data.SqlClient;
using NwndAdHcWaAPI.BusinessObject;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace NwndAdHcWaAPI.DataLayer.Base
{
     /// <summary>
     /// Base class for CategoriesDataLayer.  Do not make changes to this class,
     /// instead, put additional code in the CategoriesDataLayer class
     /// </summary>
     internal class CategoriesDataLayerBase
     {
         // constructor
         internal CategoriesDataLayerBase()
         {
         }

         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         internal static async Task<Categories> SelectByPrimaryKeyAsync(int categoryID)
         {
              Categories objCategories = null;
              string dynamicSqlScript = CategoriesSql.SelectByPrimaryKey();

              using (SqlConnection connection = new SqlConnection(AppSettings.GetConnectionString()))
              {
                  connection.Open();

                  using (SqlCommand command = new SqlCommand(dynamicSqlScript, connection))
                  {
                      command.CommandType = CommandType.Text;

                      // parameters
                      command.Parameters.AddWithValue("@categoryID", categoryID);

                      using (SqlDataAdapter da = new SqlDataAdapter(command))
                      {
                          DataTable dt = new DataTable();
                          await Task.Run(() => da.Fill(dt));

                          if (dt != null)
                          {
                              if (dt.Rows.Count > 0)
                              {
                                  objCategories = CreateCategoriesFromDataRowShared(dt.Rows[0]);
                              }
                          }
                      }
                  }
              }

              return objCategories;
         }

         /// <summary>
         /// Gets the total number of records in the Categories table
         /// </summary>
         internal static async Task<int> GetRecordCountAsync()
         {
             string sql = CategoriesSql.GetRecordCount();
             return await GetRecordCountSharedAsync(null, null, null, false, sql);
         }

         internal static async Task<int> GetRecordCountSharedAsync(string storedProcName = null, string param = null, object paramValue = null, bool isUseStoredProc = true, string dynamicSqlScript = null)
         {
              int recordCount = 0;

              using (SqlConnection connection = new SqlConnection(AppSettings.GetConnectionString()))
              {
                  connection.Open();

                  using (SqlCommand command = new SqlCommand(dynamicSqlScript, connection))
                  {
                      command.CommandType = CommandType.Text;

                      if (paramValue is null)
                          paramValue = DBNull.Value;

                      using (SqlDataAdapter da = new SqlDataAdapter(command))
                      {
                          DataTable dt = new DataTable();
                          await Task.Run(() => da.Fill(dt));

                          if (dt != null)
                          {
                              if (dt.Rows.Count > 0)
                              {
                                  recordCount = (int)dt.Rows[0]["RecordCount"];
                              }
                          }
                      }
                  }
              }

              return recordCount;
         }

         /// <summary>
         /// Gets the total number of records in the Categories table based on search parameters
         /// </summary>
         internal static async Task<int> GetRecordCountDynamicWhereAsync(int? categoryID, string categoryName)
         {
              int recordCount = 0;
              string dynamicSqlScript = CategoriesSql.GetRecordCountDynamicWhere();

              using (SqlConnection connection = new SqlConnection(AppSettings.GetConnectionString()))
              {
                  connection.Open();

                  using (SqlCommand command = new SqlCommand(dynamicSqlScript, connection))
                  {
                      command.CommandType = CommandType.Text;

                      // search parameters
                      AddSearchCommandParamsShared(command, categoryID, categoryName);

                      using (SqlDataAdapter da = new SqlDataAdapter(command))
                      {
                          DataTable dt = new DataTable();
                          await Task.Run(() => da.Fill(dt));

                          if (dt != null)
                          {
                              if (dt.Rows.Count > 0)
                              {
                                  recordCount = (int)dt.Rows[0]["RecordCount"];
                              }
                          }
                      }
                  }
              }

              return recordCount;
         }

         /// <summary>
         /// Selects Categories records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of rows)
         /// </summary>
         internal static async Task<List<Categories>> SelectSkipAndTakeAsync(string sortByExpression, int startRowIndex, int rows)
         {
             string dynamicSqlScript = CategoriesSql.SelectSkipAndTake();
             return await SelectSharedAsync(dynamicSqlScript, null, null, sortByExpression, startRowIndex, rows);
         }

         /// <summary>
         /// Selects Categories records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of records) based on search parameters
         /// </summary>
         internal static async Task<List<Categories>> SelectSkipAndTakeDynamicWhereAsync(int? categoryID, string categoryName, string sortByExpression, int startRowIndex, int rows)
         {
              List<Categories> objCategoriesCol = null;
              string dynamicSqlScript = CategoriesSql.SelectSkipAndTakeDynamicWhere();

              using (SqlConnection connection = new SqlConnection(AppSettings.GetConnectionString()))
              {
                  connection.Open();

                  using (SqlCommand command = new SqlCommand(dynamicSqlScript, connection))
                  {
                      command.CommandType = CommandType.Text;

                      // select, skip, take, sort parameters
                      command.Parameters.AddWithValue("@start", startRowIndex);
                      command.Parameters.AddWithValue("@numberOfRows", rows);
                      command.Parameters.AddWithValue("@sortByExpression", sortByExpression);

                      // search parameters
                      AddSearchCommandParamsShared(command, categoryID, categoryName);

                      using (SqlDataAdapter da = new SqlDataAdapter(command))
                      {
                          DataTable dt = new DataTable();
                          await Task.Run(() => da.Fill(dt));

                          if (dt != null)
                          {
                              if (dt.Rows.Count > 0)
                              {
                                  objCategoriesCol = new List<Categories>();

                                  foreach (DataRow dr in dt.Rows)
                                  {
                                      Categories objCategories = CreateCategoriesFromDataRowShared(dr);
                                      objCategoriesCol.Add(objCategories);
                                  }
                              }
                          }
                      }
                  }
              }

              return objCategoriesCol;
         }

         /// <summary>
         /// Selects all Categories
         /// </summary>
         internal static async Task<List<Categories>> SelectAllAsync()
         {
             string dynamicSqlScript = CategoriesSql.SelectAll();
             return await SelectSharedAsync(dynamicSqlScript, String.Empty, null);
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Categories.
         /// </summary>
         internal static async Task<List<Categories>> SelectAllDynamicWhereAsync(int? categoryID, string categoryName)
         {
              List<Categories> objCategoriesCol = null;
              string dynamicSqlScript = CategoriesSql.SelectAllDynamicWhere();

              using (SqlConnection connection = new SqlConnection(AppSettings.GetConnectionString()))
              {
                  connection.Open();

                  using (SqlCommand command = new SqlCommand(dynamicSqlScript, connection))
                  {
                      command.CommandType = CommandType.Text;

                      // search parameters
                      AddSearchCommandParamsShared(command, categoryID, categoryName);

                      using (SqlDataAdapter da = new SqlDataAdapter(command))
                      {
                          DataTable dt = new DataTable();
                          await Task.Run(() => da.Fill(dt));

                          if (dt != null)
                          {
                              if (dt.Rows.Count > 0)
                              {
                                  objCategoriesCol = new List<Categories>();

                                  foreach (DataRow dr in dt.Rows)
                                  {
                                      Categories objCategories = CreateCategoriesFromDataRowShared(dr);
                                      objCategoriesCol.Add(objCategories);
                                  }
                              }
                          }
                      }
                  }
              }

              return objCategoriesCol;
         }

         /// <summary>
         /// Selects CategoryID and CategoryName columns for use with a DropDownList web control
         /// </summary>
         internal static async Task<List<Categories>> SelectCategoriesDropDownListDataAsync()
         {
              List<Categories> objCategoriesCol = null;
              string dynamicSqlScript = CategoriesSql.SelectCategoriesDropDownListData();

              using (SqlConnection connection = new SqlConnection(AppSettings.GetConnectionString()))
              {
                  connection.Open();

                  using (SqlCommand command = new SqlCommand(dynamicSqlScript, connection))
                  {
                      command.CommandType = CommandType.Text;

                      using (SqlDataAdapter da = new SqlDataAdapter(command))
                      {
                          DataTable dt = new DataTable();
                          await Task.Run(() => da.Fill(dt));

                          if (dt != null)
                          {
                              if (dt.Rows.Count > 0)
                              {
                                  objCategoriesCol = new List<Categories>();

                                  foreach (DataRow dr in dt.Rows)
                                  {
                                      Categories objCategories = new Categories();
                                      objCategories.CategoryID = (int)dr["CategoryID"];
                                      objCategories.CategoryName = (string)(dr["CategoryName"]);

                                      objCategoriesCol.Add(objCategories);
                                  }
                              }
                          }
                      }
                  }
              }

              return objCategoriesCol;
         }

         internal static async Task<List<Categories>> SelectSharedAsync(string dynamicSqlScript, string param, object paramValue, string sortByExpression = null, int? startRowIndex = null, int? rows = null)
         {
              List<Categories> objCategoriesCol = null;

              using (SqlConnection connection = new SqlConnection(AppSettings.GetConnectionString()))
              {
                  connection.Open();

                  using (SqlCommand command = new SqlCommand(dynamicSqlScript, connection))
                  {
                      command.CommandType = CommandType.Text;

                      if (paramValue is null)
                          paramValue = DBNull.Value;

                      // select, skip, take, sort parameters
                      if (!String.IsNullOrEmpty(sortByExpression) && startRowIndex != null && rows != null)
                      {
                          command.Parameters.AddWithValue("@start", startRowIndex.Value);
                          command.Parameters.AddWithValue("@numberOfRows", rows.Value);
                          command.Parameters.AddWithValue("@sortByExpression", sortByExpression);
                      }

                      using (SqlDataAdapter da = new SqlDataAdapter(command))
                      {
                          DataTable dt = new DataTable();
                          await Task.Run(() => da.Fill(dt));

                          if (dt != null)
                          {
                              if (dt.Rows.Count > 0)
                              {
                                  objCategoriesCol = new List<Categories>();

                                  foreach (DataRow dr in dt.Rows)
                                  {
                                      Categories objCategories = CreateCategoriesFromDataRowShared(dr);
                                      objCategoriesCol.Add(objCategories);
                                  }
                              }
                          }
                      }
                  }
              }

              return objCategoriesCol;
         }

         /// <summary>
         /// Inserts a record
         /// </summary>
         internal static async Task<int> InsertAsync(Categories objCategories)
         {
             return  await InsertUpdateAsync(objCategories, false);
         }

         /// <summary>
         /// Updates a record
         /// </summary>
         internal static async Task UpdateAsync(Categories objCategories)
         {
             await InsertUpdateAsync(objCategories, true);
         }

         private static async Task<int> InsertUpdateAsync(Categories objCategories, bool isUpdate)
         {
              int newlyCreatedCategoryID = objCategories.CategoryID;

              string dynamicSqlScript = String.Empty;

              if (isUpdate)
                  dynamicSqlScript = CategoriesSql.Update();
              else
                  dynamicSqlScript = CategoriesSql.Insert();

              object description = objCategories.Description;

              if (String.IsNullOrEmpty(objCategories.Description))
                  description = System.DBNull.Value;

              using (SqlConnection connection = new SqlConnection(AppSettings.GetConnectionString()))
              {
                  connection.Open();

                  using (SqlCommand command = new SqlCommand(dynamicSqlScript, connection))
                  {
                      command.CommandType = CommandType.Text;

                      // parameters
                      if (isUpdate)
                      {
                          // for update only
                          command.Parameters.AddWithValue("@categoryID", objCategories.CategoryID);
                      }

                      command.Parameters.AddWithValue("@categoryName", objCategories.CategoryName);
                      command.Parameters.AddWithValue("@description", description);

                      if (isUpdate)
                          await command.ExecuteNonQueryAsync();
                      else
                          newlyCreatedCategoryID = (int)command.ExecuteScalar();
                  }
              }

              return newlyCreatedCategoryID;
         }

         /// <summary>
         /// Deletes a record based on primary key(s)
         /// </summary>
         internal static async Task DeleteAsync(int categoryID)
         {
              string dynamicSqlScript = CategoriesSql.Delete();

              using (SqlConnection connection = new SqlConnection(AppSettings.GetConnectionString()))
              {
                  connection.Open();

                  using (SqlCommand command = new SqlCommand(dynamicSqlScript, connection))
                  {
                      command.CommandType = CommandType.Text;

                      // parameters
                      command.Parameters.AddWithValue("@categoryID", categoryID);

                      // execute
                      await command.ExecuteNonQueryAsync();
                  }
              }
         }

         /// <summary>
         /// Adds search parameters to the Command object
         /// </summary>
         private static void AddSearchCommandParamsShared(SqlCommand command, int? categoryID, string categoryName)
         {
              if(categoryID != null)
                  command.Parameters.AddWithValue("@categoryID", categoryID);
              else
                  command.Parameters.AddWithValue("@categoryID", System.DBNull.Value);

              if(!String.IsNullOrEmpty(categoryName))
                  command.Parameters.AddWithValue("@categoryName", categoryName);
              else
                  command.Parameters.AddWithValue("@categoryName", System.DBNull.Value);

         }

         /// <summary>
         /// Creates a Categories object from the passed data row
         /// </summary>
         private static Categories CreateCategoriesFromDataRowShared(DataRow dr)
         {
             Categories objCategories = new Categories();

             objCategories.CategoryID = (int)dr["CategoryID"];
             objCategories.CategoryName = dr["CategoryName"].ToString();

             if (dr["Description"] != System.DBNull.Value)
                 objCategories.Description = dr["Description"].ToString();
             else
                 objCategories.Description = null;

             return objCategories;
         }
     }
}
