using System;
using System.Data;
using System.Data.SqlClient;
using NwndAdHcWaAPI.BusinessObject;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace NwndAdHcWaAPI.DataLayer.Base
{
     /// <summary>
     /// Base class for SuppliersDataLayer.  Do not make changes to this class,
     /// instead, put additional code in the SuppliersDataLayer class
     /// </summary>
     internal class SuppliersDataLayerBase
     {
         // constructor
         internal SuppliersDataLayerBase()
         {
         }

         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         internal static async Task<Suppliers> SelectByPrimaryKeyAsync(int supplierID)
         {
              Suppliers objSuppliers = null;
              string dynamicSqlScript = SuppliersSql.SelectByPrimaryKey();

              using (SqlConnection connection = new SqlConnection(AppSettings.GetConnectionString()))
              {
                  connection.Open();

                  using (SqlCommand command = new SqlCommand(dynamicSqlScript, connection))
                  {
                      command.CommandType = CommandType.Text;

                      // parameters
                      command.Parameters.AddWithValue("@supplierID", supplierID);

                      using (SqlDataAdapter da = new SqlDataAdapter(command))
                      {
                          DataTable dt = new DataTable();
                          await Task.Run(() => da.Fill(dt));

                          if (dt != null)
                          {
                              if (dt.Rows.Count > 0)
                              {
                                  objSuppliers = CreateSuppliersFromDataRowShared(dt.Rows[0]);
                              }
                          }
                      }
                  }
              }

              return objSuppliers;
         }

         /// <summary>
         /// Gets the total number of records in the Suppliers table
         /// </summary>
         internal static async Task<int> GetRecordCountAsync()
         {
             string sql = SuppliersSql.GetRecordCount();
             return await GetRecordCountSharedAsync(null, null, null, false, sql);
         }

         internal static async Task<int> GetRecordCountSharedAsync(string storedProcName = null, string param = null, object paramValue = null, bool isUseStoredProc = true, string dynamicSqlScript = null)
         {
              int recordCount = 0;

              using (SqlConnection connection = new SqlConnection(AppSettings.GetConnectionString()))
              {
                  connection.Open();

                  using (SqlCommand command = new SqlCommand(dynamicSqlScript, connection))
                  {
                      command.CommandType = CommandType.Text;

                      if (paramValue is null)
                          paramValue = DBNull.Value;

                      using (SqlDataAdapter da = new SqlDataAdapter(command))
                      {
                          DataTable dt = new DataTable();
                          await Task.Run(() => da.Fill(dt));

                          if (dt != null)
                          {
                              if (dt.Rows.Count > 0)
                              {
                                  recordCount = (int)dt.Rows[0]["RecordCount"];
                              }
                          }
                      }
                  }
              }

              return recordCount;
         }

         /// <summary>
         /// Gets the total number of records in the Suppliers table based on search parameters
         /// </summary>
         internal static async Task<int> GetRecordCountDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax)
         {
              int recordCount = 0;
              string dynamicSqlScript = SuppliersSql.GetRecordCountDynamicWhere();

              using (SqlConnection connection = new SqlConnection(AppSettings.GetConnectionString()))
              {
                  connection.Open();

                  using (SqlCommand command = new SqlCommand(dynamicSqlScript, connection))
                  {
                      command.CommandType = CommandType.Text;

                      // search parameters
                      AddSearchCommandParamsShared(command, supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);

                      using (SqlDataAdapter da = new SqlDataAdapter(command))
                      {
                          DataTable dt = new DataTable();
                          await Task.Run(() => da.Fill(dt));

                          if (dt != null)
                          {
                              if (dt.Rows.Count > 0)
                              {
                                  recordCount = (int)dt.Rows[0]["RecordCount"];
                              }
                          }
                      }
                  }
              }

              return recordCount;
         }

         /// <summary>
         /// Selects Suppliers records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of rows)
         /// </summary>
         internal static async Task<List<Suppliers>> SelectSkipAndTakeAsync(string sortByExpression, int startRowIndex, int rows)
         {
             string dynamicSqlScript = SuppliersSql.SelectSkipAndTake();
             return await SelectSharedAsync(dynamicSqlScript, null, null, sortByExpression, startRowIndex, rows);
         }

         /// <summary>
         /// Selects Suppliers records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of records) based on search parameters
         /// </summary>
         internal static async Task<List<Suppliers>> SelectSkipAndTakeDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax, string sortByExpression, int startRowIndex, int rows)
         {
              List<Suppliers> objSuppliersCol = null;
              string dynamicSqlScript = SuppliersSql.SelectSkipAndTakeDynamicWhere();

              using (SqlConnection connection = new SqlConnection(AppSettings.GetConnectionString()))
              {
                  connection.Open();

                  using (SqlCommand command = new SqlCommand(dynamicSqlScript, connection))
                  {
                      command.CommandType = CommandType.Text;

                      // select, skip, take, sort parameters
                      command.Parameters.AddWithValue("@start", startRowIndex);
                      command.Parameters.AddWithValue("@numberOfRows", rows);
                      command.Parameters.AddWithValue("@sortByExpression", sortByExpression);

                      // search parameters
                      AddSearchCommandParamsShared(command, supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);

                      using (SqlDataAdapter da = new SqlDataAdapter(command))
                      {
                          DataTable dt = new DataTable();
                          await Task.Run(() => da.Fill(dt));

                          if (dt != null)
                          {
                              if (dt.Rows.Count > 0)
                              {
                                  objSuppliersCol = new List<Suppliers>();

                                  foreach (DataRow dr in dt.Rows)
                                  {
                                      Suppliers objSuppliers = CreateSuppliersFromDataRowShared(dr);
                                      objSuppliersCol.Add(objSuppliers);
                                  }
                              }
                          }
                      }
                  }
              }

              return objSuppliersCol;
         }

         /// <summary>
         /// Selects all Suppliers
         /// </summary>
         internal static async Task<List<Suppliers>> SelectAllAsync()
         {
             string dynamicSqlScript = SuppliersSql.SelectAll();
             return await SelectSharedAsync(dynamicSqlScript, String.Empty, null);
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Suppliers.
         /// </summary>
         internal static async Task<List<Suppliers>> SelectAllDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax)
         {
              List<Suppliers> objSuppliersCol = null;
              string dynamicSqlScript = SuppliersSql.SelectAllDynamicWhere();

              using (SqlConnection connection = new SqlConnection(AppSettings.GetConnectionString()))
              {
                  connection.Open();

                  using (SqlCommand command = new SqlCommand(dynamicSqlScript, connection))
                  {
                      command.CommandType = CommandType.Text;

                      // search parameters
                      AddSearchCommandParamsShared(command, supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);

                      using (SqlDataAdapter da = new SqlDataAdapter(command))
                      {
                          DataTable dt = new DataTable();
                          await Task.Run(() => da.Fill(dt));

                          if (dt != null)
                          {
                              if (dt.Rows.Count > 0)
                              {
                                  objSuppliersCol = new List<Suppliers>();

                                  foreach (DataRow dr in dt.Rows)
                                  {
                                      Suppliers objSuppliers = CreateSuppliersFromDataRowShared(dr);
                                      objSuppliersCol.Add(objSuppliers);
                                  }
                              }
                          }
                      }
                  }
              }

              return objSuppliersCol;
         }

         /// <summary>
         /// Selects SupplierID and CompanyName columns for use with a DropDownList web control
         /// </summary>
         internal static async Task<List<Suppliers>> SelectSuppliersDropDownListDataAsync()
         {
              List<Suppliers> objSuppliersCol = null;
              string dynamicSqlScript = SuppliersSql.SelectSuppliersDropDownListData();

              using (SqlConnection connection = new SqlConnection(AppSettings.GetConnectionString()))
              {
                  connection.Open();

                  using (SqlCommand command = new SqlCommand(dynamicSqlScript, connection))
                  {
                      command.CommandType = CommandType.Text;

                      using (SqlDataAdapter da = new SqlDataAdapter(command))
                      {
                          DataTable dt = new DataTable();
                          await Task.Run(() => da.Fill(dt));

                          if (dt != null)
                          {
                              if (dt.Rows.Count > 0)
                              {
                                  objSuppliersCol = new List<Suppliers>();

                                  foreach (DataRow dr in dt.Rows)
                                  {
                                      Suppliers objSuppliers = new Suppliers();
                                      objSuppliers.SupplierID = (int)dr["SupplierID"];
                                      objSuppliers.CompanyName = (string)(dr["CompanyName"]);

                                      objSuppliersCol.Add(objSuppliers);
                                  }
                              }
                          }
                      }
                  }
              }

              return objSuppliersCol;
         }

         internal static async Task<List<Suppliers>> SelectSharedAsync(string dynamicSqlScript, string param, object paramValue, string sortByExpression = null, int? startRowIndex = null, int? rows = null)
         {
              List<Suppliers> objSuppliersCol = null;

              using (SqlConnection connection = new SqlConnection(AppSettings.GetConnectionString()))
              {
                  connection.Open();

                  using (SqlCommand command = new SqlCommand(dynamicSqlScript, connection))
                  {
                      command.CommandType = CommandType.Text;

                      if (paramValue is null)
                          paramValue = DBNull.Value;

                      // select, skip, take, sort parameters
                      if (!String.IsNullOrEmpty(sortByExpression) && startRowIndex != null && rows != null)
                      {
                          command.Parameters.AddWithValue("@start", startRowIndex.Value);
                          command.Parameters.AddWithValue("@numberOfRows", rows.Value);
                          command.Parameters.AddWithValue("@sortByExpression", sortByExpression);
                      }

                      using (SqlDataAdapter da = new SqlDataAdapter(command))
                      {
                          DataTable dt = new DataTable();
                          await Task.Run(() => da.Fill(dt));

                          if (dt != null)
                          {
                              if (dt.Rows.Count > 0)
                              {
                                  objSuppliersCol = new List<Suppliers>();

                                  foreach (DataRow dr in dt.Rows)
                                  {
                                      Suppliers objSuppliers = CreateSuppliersFromDataRowShared(dr);
                                      objSuppliersCol.Add(objSuppliers);
                                  }
                              }
                          }
                      }
                  }
              }

              return objSuppliersCol;
         }

         /// <summary>
         /// Inserts a record
         /// </summary>
         internal static async Task<int> InsertAsync(Suppliers objSuppliers)
         {
             return  await InsertUpdateAsync(objSuppliers, false);
         }

         /// <summary>
         /// Updates a record
         /// </summary>
         internal static async Task UpdateAsync(Suppliers objSuppliers)
         {
             await InsertUpdateAsync(objSuppliers, true);
         }

         private static async Task<int> InsertUpdateAsync(Suppliers objSuppliers, bool isUpdate)
         {
              int newlyCreatedSupplierID = objSuppliers.SupplierID;

              string dynamicSqlScript = String.Empty;

              if (isUpdate)
                  dynamicSqlScript = SuppliersSql.Update();
              else
                  dynamicSqlScript = SuppliersSql.Insert();

              object contactName = objSuppliers.ContactName;
              object contactTitle = objSuppliers.ContactTitle;
              object address = objSuppliers.Address;
              object city = objSuppliers.City;
              object region1 = objSuppliers.Region1;
              object postalCode = objSuppliers.PostalCode;
              object country = objSuppliers.Country;
              object phone = objSuppliers.Phone;
              object fax = objSuppliers.Fax;
              object homePage = objSuppliers.HomePage;

              if (String.IsNullOrEmpty(objSuppliers.ContactName))
                  contactName = System.DBNull.Value;

              if (String.IsNullOrEmpty(objSuppliers.ContactTitle))
                  contactTitle = System.DBNull.Value;

              if (String.IsNullOrEmpty(objSuppliers.Address))
                  address = System.DBNull.Value;

              if (String.IsNullOrEmpty(objSuppliers.City))
                  city = System.DBNull.Value;

              if (String.IsNullOrEmpty(objSuppliers.Region1))
                  region1 = System.DBNull.Value;

              if (String.IsNullOrEmpty(objSuppliers.PostalCode))
                  postalCode = System.DBNull.Value;

              if (String.IsNullOrEmpty(objSuppliers.Country))
                  country = System.DBNull.Value;

              if (String.IsNullOrEmpty(objSuppliers.Phone))
                  phone = System.DBNull.Value;

              if (String.IsNullOrEmpty(objSuppliers.Fax))
                  fax = System.DBNull.Value;

              if (String.IsNullOrEmpty(objSuppliers.HomePage))
                  homePage = System.DBNull.Value;

              using (SqlConnection connection = new SqlConnection(AppSettings.GetConnectionString()))
              {
                  connection.Open();

                  using (SqlCommand command = new SqlCommand(dynamicSqlScript, connection))
                  {
                      command.CommandType = CommandType.Text;

                      // parameters
                      if (isUpdate)
                      {
                          // for update only
                          command.Parameters.AddWithValue("@supplierID", objSuppliers.SupplierID);
                      }

                      command.Parameters.AddWithValue("@companyName", objSuppliers.CompanyName);
                      command.Parameters.AddWithValue("@contactName", contactName);
                      command.Parameters.AddWithValue("@contactTitle", contactTitle);
                      command.Parameters.AddWithValue("@address", address);
                      command.Parameters.AddWithValue("@city", city);
                      command.Parameters.AddWithValue("@region1", region1);
                      command.Parameters.AddWithValue("@postalCode", postalCode);
                      command.Parameters.AddWithValue("@country", country);
                      command.Parameters.AddWithValue("@phone", phone);
                      command.Parameters.AddWithValue("@fax", fax);
                      command.Parameters.AddWithValue("@homePage", homePage);

                      if (isUpdate)
                          await command.ExecuteNonQueryAsync();
                      else
                          newlyCreatedSupplierID = (int)command.ExecuteScalar();
                  }
              }

              return newlyCreatedSupplierID;
         }

         /// <summary>
         /// Deletes a record based on primary key(s)
         /// </summary>
         internal static async Task DeleteAsync(int supplierID)
         {
              string dynamicSqlScript = SuppliersSql.Delete();

              using (SqlConnection connection = new SqlConnection(AppSettings.GetConnectionString()))
              {
                  connection.Open();

                  using (SqlCommand command = new SqlCommand(dynamicSqlScript, connection))
                  {
                      command.CommandType = CommandType.Text;

                      // parameters
                      command.Parameters.AddWithValue("@supplierID", supplierID);

                      // execute
                      await command.ExecuteNonQueryAsync();
                  }
              }
         }

         /// <summary>
         /// Adds search parameters to the Command object
         /// </summary>
         private static void AddSearchCommandParamsShared(SqlCommand command, int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax)
         {
              if(supplierID != null)
                  command.Parameters.AddWithValue("@supplierID", supplierID);
              else
                  command.Parameters.AddWithValue("@supplierID", System.DBNull.Value);

              if(!String.IsNullOrEmpty(companyName))
                  command.Parameters.AddWithValue("@companyName", companyName);
              else
                  command.Parameters.AddWithValue("@companyName", System.DBNull.Value);

              if(!String.IsNullOrEmpty(contactName))
                  command.Parameters.AddWithValue("@contactName", contactName);
              else
                  command.Parameters.AddWithValue("@contactName", System.DBNull.Value);

              if(!String.IsNullOrEmpty(contactTitle))
                  command.Parameters.AddWithValue("@contactTitle", contactTitle);
              else
                  command.Parameters.AddWithValue("@contactTitle", System.DBNull.Value);

              if(!String.IsNullOrEmpty(address))
                  command.Parameters.AddWithValue("@address", address);
              else
                  command.Parameters.AddWithValue("@address", System.DBNull.Value);

              if(!String.IsNullOrEmpty(city))
                  command.Parameters.AddWithValue("@city", city);
              else
                  command.Parameters.AddWithValue("@city", System.DBNull.Value);

              if(!String.IsNullOrEmpty(region1))
                  command.Parameters.AddWithValue("@region1", region1);
              else
                  command.Parameters.AddWithValue("@region1", System.DBNull.Value);

              if(!String.IsNullOrEmpty(postalCode))
                  command.Parameters.AddWithValue("@postalCode", postalCode);
              else
                  command.Parameters.AddWithValue("@postalCode", System.DBNull.Value);

              if(!String.IsNullOrEmpty(country))
                  command.Parameters.AddWithValue("@country", country);
              else
                  command.Parameters.AddWithValue("@country", System.DBNull.Value);

              if(!String.IsNullOrEmpty(phone))
                  command.Parameters.AddWithValue("@phone", phone);
              else
                  command.Parameters.AddWithValue("@phone", System.DBNull.Value);

              if(!String.IsNullOrEmpty(fax))
                  command.Parameters.AddWithValue("@fax", fax);
              else
                  command.Parameters.AddWithValue("@fax", System.DBNull.Value);

         }

         /// <summary>
         /// Creates a Suppliers object from the passed data row
         /// </summary>
         private static Suppliers CreateSuppliersFromDataRowShared(DataRow dr)
         {
             Suppliers objSuppliers = new Suppliers();

             objSuppliers.SupplierID = (int)dr["SupplierID"];
             objSuppliers.CompanyName = dr["CompanyName"].ToString();

             if (dr["ContactName"] != System.DBNull.Value)
                 objSuppliers.ContactName = dr["ContactName"].ToString();
             else
                 objSuppliers.ContactName = null;

             if (dr["ContactTitle"] != System.DBNull.Value)
                 objSuppliers.ContactTitle = dr["ContactTitle"].ToString();
             else
                 objSuppliers.ContactTitle = null;

             if (dr["Address"] != System.DBNull.Value)
                 objSuppliers.Address = dr["Address"].ToString();
             else
                 objSuppliers.Address = null;

             if (dr["City"] != System.DBNull.Value)
                 objSuppliers.City = dr["City"].ToString();
             else
                 objSuppliers.City = null;

             if (dr["Region1"] != System.DBNull.Value)
                 objSuppliers.Region1 = dr["Region1"].ToString();
             else
                 objSuppliers.Region1 = null;

             if (dr["PostalCode"] != System.DBNull.Value)
                 objSuppliers.PostalCode = dr["PostalCode"].ToString();
             else
                 objSuppliers.PostalCode = null;

             if (dr["Country"] != System.DBNull.Value)
                 objSuppliers.Country = dr["Country"].ToString();
             else
                 objSuppliers.Country = null;

             if (dr["Phone"] != System.DBNull.Value)
                 objSuppliers.Phone = dr["Phone"].ToString();
             else
                 objSuppliers.Phone = null;

             if (dr["Fax"] != System.DBNull.Value)
                 objSuppliers.Fax = dr["Fax"].ToString();
             else
                 objSuppliers.Fax = null;

             if (dr["HomePage"] != System.DBNull.Value)
                 objSuppliers.HomePage = dr["HomePage"].ToString();
             else
                 objSuppliers.HomePage = null;

             return objSuppliers;
         }
     }
}
